#!/usr/bin/python3
# -*- mode:python; coding:utf-8; -*-
import os


def getCPName():
    # cPanel
    if os.path.isfile('/usr/local/cpanel/cpanel'):
        return "cPanel"
    # DirectAdmin
    if os.path.isfile('/usr/local/directadmin/directadmin') or \
       os.path.isfile('/usr/local/directadmin/custombuild/build'):
        return "DirectAdmin"
    # InterWorx
    if os.path.isfile('/usr/local/interworx/iworx.ini'):
        return "InterWorx"
    # ISPManager
    if os.path.isfile('/usr/local/ispmgr/bin/ispmgr'):
        return "ISPManager"
    # Plesk
    if os.path.isfile('/usr/local/psa/version'):
        return "Plesk"
    # Unknown panel / No panel
    return "Unknown"